##************************** 
## The following code corresponds to the Introduction to ggplot2 workshop
## given by University of Virginia Library's StatLab
## March 18, 2015
# Compiled by Caitlin I. Steiner
# Used code from "R Graphics Cookbook" by Winston Chang 
# and other sources listed on corresponding slides
##**************************

#install.packages("ggplot2")
library("ggplot2")
#install.packages("gcookbook")
library("gcookbook")

##************************** 
## Simple Line Graph
##**************************
# Using BOD dataset
head(BOD)

##*** You want to create a line graph.
# In Base R would need to run
plot(BOD$Time, BOD$demand, type="l")  #is a lowercase L not a 1
# or plot(BOD$demand~BOD$Time, type="l") 
# In ggplot2
ggplot(BOD, aes(x=Time, y=demand)) +
  geom_line()

# change the appearance of the line in the line graph
# (default is solid black line)
ggplot(BOD, aes(x=Time, y=demand)) +
  geom_line(linetype="dashed")
# change line size
ggplot(BOD, aes(x=Time, y=demand)) +
  geom_line(linetype="dashed", size=1)
# change line colour
ggplot(BOD, aes(x=Time, y=demand)) +
  geom_line(linetype="dashed", size=1, colour="blue")
# equivalent  to the following in Base R
plot(BOD$Time, BOD$demand, type="l",col="blue",lty=2, lwd=1)


##************************** 
## Simple Scatterplot
##**************************
# Just using two columns from the dataset heightweight
head(heightweight[,c("ageYear","heightIn")])

##*** You want to create a scatterplot
# In Base R would need to run
plot(heightweight$ageYear, heightweight$heightIn)
# or plot(height$heightIn~height$ageYear)
# In ggplot2
ggplot(heightweight, aes(x=ageYear, y=heightIn)) +
  geom_point()

# change the shape of the points from the defualt solid circles
# to hallow circles
ggplot(heightweight, aes(x=ageYear, y=heightIn)) +
  geom_point(shape=21)
# In base R this is equivalent to 
plot(heightweight$ageYear, heightweight$heightIn, pch=16)

# change the size of the points from the default 2
ggplot(heightweight, aes(x=ageYear, y=heightIn)) +
  geom_point(size=1.5)
# change color ("outside color")
ggplot(heightweight, aes(x=ageYear, y=heightIn)) +
  geom_point(colour="darkred")
# change fill ("inside color")
ggplot(heightweight, aes(x=ageYear, y=heightIn)) +
  geom_point(fill="pink")


##************************** 
## Multiple Line Graph
##**************************
# Using the uspopage dataset

##*** You want to make a line graph with more than one line
# To do this one must map another (discrete) variable to either
# colour or linetype

# Categories of AgeGroup indicated by different line types
ggplot(uspopage, aes(x=Year, y=Thousands, linetype=AgeGroup)) +
  geom_line()
# equivalent to the following in Base R
plot(uspopage$Year[uspopage$AgeGroup=="<5"], uspopage$Thousands[uspopage$AgeGroup=="<5"], type="l", ylim=c(min(uspopage$Thousands), max(uspopage$Thousands)))
lines(uspopage$Year[uspopage$AgeGroup=="5-14"], uspopage$Thousands[uspopage$AgeGroup=="5-14"], lty=2 )
lines(uspopage$Year[uspopage$AgeGroup=="15-24"], uspopage$Thousands[uspopage$AgeGroup=="15-24"], lty=3 )
lines(uspopage$Year[uspopage$AgeGroup=="25-34"], uspopage$Thousands[uspopage$AgeGroup=="25-34"], lty=4 )
legend('topleft',c("<5","5-14","15-24","25-34"), lty=c(1,2,3,4))
# etc...

# Categories of AgeGroup indicated by different colors
ggplot(uspopage, aes(x=Year, y=Thousands, colour=AgeGroup)) +
  geom_line()


##************************** 
## Multiple Group Scatterplot
##**************************
# Using the heightweight dataset

##*** You want to create a scatterplot with multiple groups
# To do this again we can map a (continuous or discrete) variable
# to shape or colour

# Discrete case
# Categories of sex indicated by different colors
ggplot(heightweight, aes(x=ageYear, y=heightIn, colour=sex)) +
  geom_point()
# Categories of sex indicated by different shapes
ggplot(heightweight, aes(x=ageYear, y=heightIn, shape=sex)) +
  geom_point()
# Categories of sex indicated by both color and shape
ggplot(heightweight, aes(x=ageYear, y=heightIn, shape=sex, colour=sex)) +
  geom_point()

# Continuous Case
# Represent continuous variable weightLb using color
ggplot(heightweight, aes(x=ageYear, y=heightIn, colour=weightLb)) +
  geom_point()
# Represent continuous variable weightLb using size
ggplot(heightweight, aes(x=ageYear, y=heightIn, size=weightLb)) +
  geom_point()


##************************** 
## Adding Fitted REgression Model Lines to Scatterplot
##**************************
# Using the heightweight dataset

##*** You want to add lines from a fitted regression model
sp <- ggplot(heightweight, aes(x=ageYear, y=heightIn)) +
  geom_point()
sp

# Add a linear model to scatterplot (adds 95% confidence region)
sp + stat_smooth(method=lm)
# Add 99% confidence region
sp + stat_smooth(method=lm, level=0.99)
# Have no confidence region
sp + stat_smooth(method=lm, se=FALSE)
# Add a loess (locally weighted polynomial curve)
sp + stat_smooth()
# which is equivalent to as loess is default
sp + stat_smooth(method=loess)


##*** You have already created a fitted regression model 
# and want to plot lines for that on graph
# This is fitted model
model <- lm(heightIn ~ ageYear + I(ageYear^2), heightweight)
# Create dataframe with ageYear column, interpolating across range
xmin <- min(heightweight$ageYear)
xmax <- max(heightweight$ageYear)
predicted <- data.frame(ageYear=seq(xmin, xmax, length.out=100))
# Calculate predicted values for heightIn
predicted$heightIn <- predict(model, predicted)
head(predicted)
# Add model to graph
sp + geom_line(data=predicted, size=1)


##************************** 
## Subplotting Data (Facets)
##**************************
# Using mpg dataset

##*** You want to plot subsets of data in separate panels
p <- ggplot(mpg,aes(x=displ, y=hwy)) + geom_point()
p
# Faceted by drv, in vertically arranged subpanels
p + facet_grid(drv ~ .)
# faceted by cyl, in horizontally arranged subpanels
p + facet_grid(. ~ cyl)
# Split by drv (vertical) and cycl (horizontal)
p + facet_grid(drv ~ cyl)

# Using facet_wrap() instead
p + facet_wrap( ~ class)  #Note there is nothing before the tilde

# Change the default number of subplots (default is square fitting all)
p + facet_wrap( ~ class, nrow=2)
p + facet_wrap( ~ class, ncol=4)


##*** You want subplots with different ranges or items on their axes
p <- ggplot(mpg,aes(x=displ, y=hwy)) +
  geom_point()
# with free y scales
p + facet_grid(drv ~ cyl, scales="free_y")
# with free x scales
p + facet_grid(drv ~ cyl, scales="free")


##************************** 
## Complex Line Graph
##**************************
# Using BOD dataset

##*** You want to create a line graph with a certain y-axis range
# The default ggplot2 y-axis range for a line graph is just enough
# to include the y-values in the data, but sometimes you want the 
# y range to start from zero
ggplot(BOD, aes(x=Time, y=demand)) +
  geom_line() +
  ylim(0,max(BOD$demand))
# same as
ggplot(BOD, aes(x=Time, y=demand)) +
  geom_line() +
  expand_limits(y=0)
#In Base R you would need to have
plot(BOD$Time, BOD$demand, type="l",ylim=c(0,max(BOD$demand)))


##*** You want to add points to the line graph, 
# we would just add another geometric object to the plot
ggplot(BOD, aes(x=Time, y=demand)) +
  geom_line() +
  geom_point()
# In Base R you would need to have
plot(BOD$Time, BOD$demand, type="l")
points(BOD$Time, BOD$demand)


##************************** 
## Dealing with Overplotting
##**************************
# Using dataset diamonds

##*** Say you have points that obscure one another
sp <- ggplot(diamonds, aes(x=carat, y=price))
sp + geom_point()
# We can make the points so that they are semitransparent
sp + geom_point(alpha=0.1)
sp + geom_point(alpha=0.01)

# Bin the data into rectangles
sp + stat_bin2d()
sp + stat_bin2d(bins=50) +
  scale_fill_gradient(low="lightblue",high="red",limits=c(0,6000))


##**************************
## Simple Bar Graph
##**************************
# Using pg_mean dataset

##*** You want to create a basic bar graph
ggplot(pg_mean, aes(x=group,y=weight)) +
  geom_bar(stat="identity")

#change color of bars
ggplot(pg_mean, aes(x=group,y=weight)) +
  geom_bar(stat="identity",fill="lightblue", colour="black")

# Adjust bar width and spacing
barplot <- ggplot(pg_mean, aes(x=group, y=weight))
# for narrower bars
barplot + geom_bar(stat="identity", width=0.5)
# for wider bars (maximum width of 1)
barplot + geom_bar(stat="identity", width=1)


##************************** 
## Grouped Bar Graph
##**************************
# Using cabbage_exp dataset

##*** You want to group bars together by a second variable
ggplot(cabbage_exp,aes(x=Date,y=Weight,fill=Cultivar)) +
  geom_bar(stat="identity",position="dodge")

# For grouped bars, the default is to have no space between bars
# within each group.
ggplot(cabbage_exp,aes(x=Date,y=Weight,fill=Cultivar)) +
  geom_bar(stat="identity",position="dodge", width=0.5)
# can't just make bars narrower but need to shift position.
# Note position="dodge" is shorthand for position=position_dodge(0.9).
ggplot(cabbage_exp,aes(x=Date,y=Weight,fill=Cultivar)) +
  geom_bar(stat="identity",width=0.5, position=position_dodge(0.7))


##************************** 
## Stacked Bar Graph 
##**************************
# You want to make a stacked bar graph
ggplot(cabbage_exp, aes(x=Date, y=Weight, fill=Cultivar)) +
  geom_bar(stat="identity")


##************************** 
## Labeling Bar Graphs
##**************************
# Using cabbage_exp dataset

##*** You want to add labels to the bars in a bar graph
# Place text below the top of the bar
ggplot(cabbage_exp,aes(x=interaction(Date, Cultivar), y=Weight)) +
  geom_bar(stat="identity") +
  geom_text(aes(label=Weight),vjust=1.5,colour="white")

# Place text above the top of the bar
ggplot(cabbage_exp,aes(x=interaction(Date, Cultivar), y=Weight)) +
  geom_bar(stat="identity") +
  geom_text(aes(label=Weight),vjust=-0.2)

# Labels for grouped bar graphs
ggplot(cabbage_exp, aes(x=Date, y=Weight, fill=Cultivar)) +
  geom_bar(stat="identity", position="dodge") +
  geom_text(aes(label=Weight),vjust=1.5, colour="white",
            position=position_dodge(.9),size=3)

# Labels for Stacked bar graphs
#install.packages(plyr)
library(plyr)
# sort by the day and sex columns
ce <- arrange(cabbage_exp, Date, Cultivar)
# get the cumulative sum
ce <- ddply(ce, "Date", transform, label_y=cumsum(Weight))
ce
ggplot(ce, aes(x=Date, y=Weight, fill=Cultivar)) +
  geom_bar(stat="identity") +
  geom_text(aes(y=label_y, label=Weight), vjust=1.5, colour="white")


##************************** 
## Basic Histogram
##**************************
# Using faithful dataset

##*** You want to make a histogram
# In Basic R you would run
hist(faithful$waiting)
# In ggplot2 it would be (default is 30 bins)
ggplot(faithful, aes(x=waiting)) + 
  geom_histogram()

# Set the width of each bin to 5
ggplot(faithful, aes(x=waiting)) +
  geom_histogram(binwidth=5, fill="white", colour="black")

# Divide the x range into 15 bins
binsize <- diff(range(faithful$waiting))/15
ggplot(faithful, aes(x=waiting)) +
  geom_histogram(binwidth=binsize, fill="white", colour="black")

##************************** 
## Grouped Data Histogram
##**************************
# Using birthwt dataset from MASS package
#install.packages(MASS)
library(MASS)
head(birthwt)

##*** You want to make histograms of multiple groups of data
ggplot(birthwt, aes(x=bwt)) +
  geom_histogram(fill="white", color="black") +
  facet_grid(smoke~.)
# To change labels to indicate those who smoke and those that don't, we need to change the names of the factor levels
birthwt1 <- birthwt # Make a copy of the data
# Convert smoke to a factor
birthwt1$smoke <- factor(birthwt1$smoke)
levels(birthwt1$smoke)
#library(plyr)
birthwt1$smoke <- revalue(birthwt1$smoke, c("0"="No Smoke", "1"="Smoke"))
ggplot(birthwt1, aes(x=bwt)) +
  geom_histogram(fill="white", colour="black") +
  facet_grid(smoke~.)

# Graphs groups on top of one another (not stacked) and make them semitransparent
ggplot(birthwt1, aes(x=bwt, fill=smoke)) +
  geom_histogram(position="identity", alpha=0.4)


##************************** 
## Density Curve
##**************************
# Using faithful dataset

###*** You want to make a kernel density curve
ggplot(faithful, aes(x=waiting)) +
  geom_density()
# if don't like the lines along the side and bottom
ggplot(faithful, aes(x=waiting)) +
  geom_line(stat="density")

# adjust the amount of smoothing
ggplot(faithful, aes(x=waiting)) +
  geom_line(stat="density", adjust=0.25, colour="red") +
  geom_line(stat="density") +
  geom_line(stat="density", adjust=2, colour="blue")

# draw a blue polygon with geom_density, then add a line on top
ggplot(faithful, aes(x=waiting)) +
  geom_density(fill="blue", colour=NA, alpha=0.2) +
  geom_line(stat="density")

# overlay the density curve on top of the histogram by scaling down the histogram mapping
ggplot(faithful, aes(x=waiting, y=..density..)) +
  geom_histogram(fill="cornsilk", colour="grey60", size=.2) +
  geom_density()


##*** You want to make density curves of multiple groups of data
# Using the dataset birthwt from MASS package
#library(MASS)\
birthwt1<-birthwt
birthwt1$smoke<-factor(birthwt1$smoke)
ggplot(birthwt1, aes(x=bwt, colour=smoke)) +
  geom_density()

# make fill color and semitransparent instead
ggplot(birthwt1, aes(x=bwt, fill=smoke)) +
  geom_density(alpha=0.3)


##************************** 
## Boxplot
##**************************
# Using birthwt dataset from MASS package

###*** You want make boxplot (box-and-whisker)
# uses factor to convert numeric variable to discrete
ggplot(birthwt,aes(x=factor(race), y=bwt)) +
  geom_boxplot()
#In base R, would use
boxplot(birthwt$bwt~factor(birthwt$race))

# change width of the boxes
ggplot(birthwt, aes(x=factor(race), y=bwt)) +
  geom_boxplot(width=0.5)

# change outlier size and shape if possible overplotting
ggplot(birthwt, aes(x=factor(race), y=bwt)) +
  geom_boxplot(outlier.size=1.5, outlier.shape=21)

# create boxplot for just one group, provide arbitrary value for x
ggplot(birthwt, aes(x=1, y=bwt)) +
  geom_boxplot()

##*** You want to add notches to a boxplot to assess whether the medians are different
ggplot(birthwt, aes(x=factor(race), y=bwt)) +
  geom_boxplot(notch=TRUE)

##*** You want to add markers for the mean to a boxplot
ggplot(birthwt, aes(x=factor(race), y=bwt)) +
  geom_boxplot() +
  stat_summary(fun.y="mean", geom="point", shape=23, size=3, fill="white")


##************************** 
## Changing Themes
##**************************
# Using heightweight dataset
p <- ggplot(heightweight, aes(x=ageYear, y=heightIn))+
  geom_point()
p

##*** You want to use premade themes to contol the overall plot appearance
# (two themes available are grey and bw)
# default is a grey theme
p + theme_grey()
# Black and white theme
p + theme_bw()
# Can also change default for current R session using theme_set()
theme_set(theme_bw())
p
# Reset to default theme
theme_set(theme_grey())
p


##*** You want to set the title of a graph
# You can use ggtitle() or labs()
p + ggtitle("Age and Height of Schoolchildren")
# Use \n for a newline
p + ggtitle("Age and Height \nof Schoolchildren")
# equivalent to 
p + labs(title="Age and Height \nof Schoolchildren")


##*** You want to change appearance of theme elements
# To modify a theme we use a corresponding element_xx object,
# such as element_line, element_rect, and element_text
# Options for text items
p <- ggplot(heightweight, aes(x=ageYear, y=heightIn, colour=sex))+
  geom_point()
p
p + ggtitle("Title")+
  theme(axis.title.x=element_text(colour="red", size=14),
        axis.text.x=element_text(colour="blue"),
        axis.title.y=element_text(colour="red",size=14, angle=90),
        axis.text.y=element_text(colour="blue"),
        plot.title=element_text(colour="red",size=20,face="bold"))
# Options for the legend
p + theme(legend.background=element_rect(fill="grey85",colour="red", size=1),
          legend.title=element_text(colour="blue", face="bold",size=14),
          legend.text=element_text(colour="red"),
          legend.key=element_rect(colour="blue", size=0.25),
          plot.title=element_text(colour="red",size=20,face="bold"))
# Options for facets
p + facet_grid(sex~.) +
  theme(strip.background=element_rect(fill="pink"),
        strip.text.y=element_text(size=14, angle=-90, face="bold"))


##*** You want to hide the grid lines
# Hide all grid lines
p + theme(panel.grid.major=element_blank(),
          panel.grid.minor=element_blank())
# Hide vertical grid lines
p + theme(panel.grid.major.x=element_blank(),
          panel.grid.minor.x=element_blank())
# Hide horizontal grid lines
p + theme(panel.grid.major.y=element_blank(),
          panel.grid.minor.y=element_blank())


##*** You want to move the legend from the default position (on the right)
# Use the legend.position argument in the theme layer
p + theme(legend.position="top")
# Move legend inside of graphing area using legend.position 
# and legend.justification.
# Coordinate space starts at (0,0) at bottom left
# and goes to (1,1) at top right
p + theme(legend.position=c(1,0), legend.justification=c(1,0))
p + theme(legend.position=c(1,1), legend.justification=c(1,1))


##*** You want to change the legend title
# You can use labs() and change corresponding aesthetic used
p + labs(colour="Gender")


##*** You want to change the order of the items in the legend
# Use the scale_<aes used>_discrete and change limits argument
p + scale_colour_discrete(limits=c("m","f"))


##*** You want to change the labels in the legend
# Use the scale_<aes used>_discrete and change labels agrument
p + scale_colour_discrete(labels=c("female", "male"))
                          
                          
##************************** 
## Changing Axes
##**************************
# Using PlantGrowth dataset

p <- ggplot(PlantGrowth, aes(x=group, y=weight)) +
  geom_boxplot()
p

##*** You want to swap the x and y axes
# Use coord_flip()
p + coord_flip()
# This is pointless to do for scatterplots 
# as one should just switch the x and y values declared


##*** You want to change the order of the items on the categorical axis
# Set limit in scale_x_discrete() or scale_y_discrete()
p + scale_x_discrete(limits=c("trt1", "ctrl", "trt2"))


# Use heightweight dataset
##*** You want to change the text of the axis labels
# Use xlab() or ylab()
hwp <- ggplot(heightweight, aes(x=ageYear, y=heightIn, colour=sex)) +
  geom_point()
hwp
hwp + xlab("Age in years") +
  ylab("Height in inches")
                          
                  